/**
 * @(#)pms13_31.h 2006/07/11
 *
 * @author Tognon Stefano
 * @version 1.00
 *
 * This program automatically generates random prime magic squares of order 13
 * for sequence of primes starting from prime 13.
 * The square is printed in standard output, while the other information are
 * printed in standard error.
 *
 * note: some comments are in the style of Javadoc of Java language!
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307  USA.
 */

typedef unsigned long long int word;
#define DORDER 13
const int ORDER=DORDER; // square order
const int COMB=8192;    // 2 at pow of order
const word MAGIC=47395927491;  // magic constant

// number of max spaces between output values
const int MAX_BLANK=5;

// list of sequential primes numbers
const word PRIME[ORDER*ORDER]={
        853,  857,  859,  863,  877,  881,  883,  887,
        907,  911,  919,  929,  937,  941,  947,  953,  967,  971,  977,
        983,  991,  997, 1009, 1013, 1019, 1021, 1031, 1033, 1039, 1049,
       1051, 1061, 1063, 1069, 1087, 1091, 1093, 1097, 1103, 1109, 1117,
       1123, 1129, 1151, 1153, 1163, 1171, 1181, 1187, 1193, 1201, 1213,
       1217, 1223, 1229, 1231, 1237, 1249, 1259, 1277, 1279, 1283, 1289,
       1291, 1297, 1301, 1303, 1307, 1319, 1321, 1327, 1361, 1367, 1373,
       1381, 1399, 1409, 1423, 1427, 1429, 1433, 1439, 1447, 1451, 1453,
       1459, 1471, 1481, 1483, 1487, 1489, 1493, 1499, 1511, 1523, 1531,
       1543, 1549, 1553, 1559, 1567, 1571, 1579, 1583, 1597, 1601, 1607,
       1609, 1613, 1619, 1621, 1627, 1637, 1657, 1663, 1667, 1669, 1693,
       1697, 1699, 1709, 1721, 1723, 1733, 1741, 1747, 1753, 1759, 1777,
       1783, 1787, 1789, 1801, 1811, 1823, 1831, 1847, 1861, 1867, 1871,
       1873, 1877, 1879, 1889, 1901, 1907, 1913, 1931, 1933, 1949, 1951,
       1973, 1979, 1987, 1993, 1997, 1999, 2003, 2011, 2017, 2027, 2029,
       2039, 2053, 2063, 2069, 2081, 2083, 2087};

// this is a union of bits and the corresponding int value.
// if the program never produce results, probably your compiler manage this
// union in a different way. If we increase 'counter' then some bits may
// changes their value.
union unionBits {
  int counter;
  struct {
    unsigned bit0 :1;
    unsigned bit1 :1;
    unsigned bit2 :1;
    unsigned bit3 :1;
    unsigned bit4 :1;
    unsigned bit5 :1;
    unsigned bit6 :1;
    unsigned bit7 :1;
    unsigned bit8 :1;
    unsigned bit9 :1;
    unsigned bit10 :1;
    unsigned bit11 :1;
    unsigned bit12 :1;
  } bit;
};

// these defines work with odd order

#define UP_SEQ {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12}
#define DN_SEQ {12, 11, 10, 9, 8, 7, 6, 5, 4, 3, 2, 1, 0}

#define IJ(a) (i==a || j==(ORDER-a-1) || j==a || i==(ORDER-a-1))

#define POS(a) (pos[a]!=a)
#define POSES   POS(0) && POS(1) && POS(2) && POS(3) && \
                POS(4) && POS(5) && (pos[6]==6) && POS(7) && \
                POS(8) && POS(9) && POS(10) && POS(11) && \
                POS(12)

#define VALUE(a) (sq.cell[pos[a]][a])
#define VALUES  (VALUE(0)+ VALUE(1)+ VALUE(2)+ VALUE(3)+ VALUE(4)+ \
                 VALUE(5)+  VALUE(6)+ VALUE(7)+ VALUE(8)+ VALUE(9)+ \
                 VALUE(10)+ VALUE(11)+ VALUE(12))

#define IF_SWAP if (bits.bit.bit0) swap(sq, i, 0, j, 0); \
                if (bits.bit.bit1) swap(sq, i, 1, j, 1); \
                if (bits.bit.bit2) swap(sq, i, 2, j, 2); \
                if (bits.bit.bit3) swap(sq, i, 3, j, 3); \
                if (bits.bit.bit4) swap(sq, i, 4, j, 4); \
                if (bits.bit.bit5) swap(sq, i, 5, j, 5); \
                if (bits.bit.bit6) swap(sq, i, 6, j, 6); \
                if (bits.bit.bit7) swap(sq, i, 7, j, 7); \
                if (bits.bit.bit8) swap(sq, i, 8, j, 8); \
                if (bits.bit.bit9) swap(sq, i, 9, j, 9); \
                if (bits.bit.bit10) swap(sq, i, 10, j, 10); \
                if (bits.bit.bit11) swap(sq, i, 11, j, 11); \
                if (bits.bit.bit12) swap(sq, i, 12, j, 12);

#define SUM_CELL sq.cell[index[bits.bit.bit0]][0]+ \
                 sq.cell[index[bits.bit.bit1]][1]+ \
                 sq.cell[index[bits.bit.bit2]][2]+ \
                 sq.cell[index[bits.bit.bit3]][3]+ \
                 sq.cell[index[bits.bit.bit4]][4]+ \
                 sq.cell[index[bits.bit.bit5]][5]+ \
                 sq.cell[index[bits.bit.bit6]][6]+ \
                 sq.cell[index[bits.bit.bit7]][7]+ \
                 sq.cell[index[bits.bit.bit8]][8]+ \
                 sq.cell[index[bits.bit.bit9]][9]+ \
                 sq.cell[index[bits.bit.bit10]][10]+ \
                 sq.cell[index[bits.bit.bit11]][11]+ \
                 sq.cell[index[bits.bit.bit12]][12]

#define BIT_IJ (bits.bit.bit0 && IJ(0)) || \
               (bits.bit.bit1 && IJ(1)) || \
               (bits.bit.bit2 && IJ(2)) || \
               (bits.bit.bit3 && IJ(3)) || \
               (bits.bit.bit4 && IJ(4)) || \
               (bits.bit.bit5 && IJ(5)) || \
               (bits.bit.bit6 && IJ(6)) || \
               (bits.bit.bit7 && IJ(7)) || \
               (bits.bit.bit8 && IJ(8)) || \
               (bits.bit.bit9 && IJ(9)) || \
               (bits.bit.bit10 && IJ(10)) || \
               (bits.bit.bit11 && IJ(11)) || \
               (bits.bit.bit12 && IJ(12))

#define cerr_h  \
               "Usage: pms13_13 [-m.] [-n,] [-t;]" << endl \
            << "Options:" << endl \
            << " -m<mh> where mh is the method to use (default d3):" << endl \
            << "    a1: 13R 13C  2D 13R   v1\t  5 sq/min in PII 266Mhz " \
            <<                           "(average of  100 squares)" << endl \
            << "    a2: 13R 13C  2D 13R   v2\t  1 sq/min in PII 266Mhz " \
            <<                           "(average of   10 squares)" << endl \
            << "    a3: 13R 13C  2D 13R   v3\t  8 sq/min in PII 266Mhz " \
            <<                           "(average of  100 squares)" << endl \
            << "    b1:  2R  2D 12R 12C   v1\t  4 sq/min in PII 266Mhz " \
            <<                           "(average of   20 squares)" << endl \
            << "    b2:  2R  2D 12R 12C   v2\t  0 sq/min in PII 266Mhz " \
            <<                           "(average of   20 squares)" << endl \
            << "    b3:  2R  2D 12R 12C   v3\t  4 sq/min in PII 266Mhz " \
            <<                           "(average of  100 squares)" << endl \
            << "    c1: 13R 13C 1D        v1\t 18 sq/min in PII 266Mhz " \
            <<                           "(average of  100 squares)" << endl \
            << "    c2: 13R 13C 1D        v2\t 15 sq/min in PII 266Mhz " \
            <<                           "(average of  100 squares)" << endl \
            << "    c3: 13R 13C 1D        v3\t 19 sq/min in PII 266Mhz " \
            <<                           "(average of  100 squares)" << endl \
            << "    d1: 13R 13C 1D 1D 13R v1\t391 sq/min in PII 266Mhz " \
            <<                           "(average of 2000 squares)" << endl \
            << "    d2: 13R 13C 1D 1D 13R v2\tdon't give speed result" << endl \
            << "    d3: 13R 13C 1D 1D 13R v3\t430 sq/min in PII 266Mhz " \
            <<                           "(average of 4000 squares)" << endl \
            << "    d4: 13R 13C 1D 1D 13R v3\t329 sq/min in PII 266Mhz " \
            <<                           "(average of 4000 squares)" << endl \
            << "    (R=row C=column D=diagonal, v version)" << endl \
            << " -n<nu> where nu is the number of squares to generate " \
            <<          "(default 1)" << endl \
            << " -t<nn> where nn is the number of cycle for the methods that " \
            <<                         "allow time out (only c3)" << endl \
            << " -h     the help" << endl;

#define cerr_default "Invalid option: use pms13_13 -h for help" << endl;
