/**
 * @(#)pms6_7.h 2000/03/15
 *
 * @author Tognon Stefano
 * @version 1.00
 *
 * This program automatically generates random prime magic squares of order 6
 * for sequence of primes starting from prime 7.
 * The square is printed in standard output, while the other information are
 * printed in standard error.
 *
 * note: some comments are in the style of Javadoc of Java language!
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307  USA.
 */

typedef unsigned int word;
#define DORDER 6
const int ORDER=DORDER; // square order
const int COMB=64;      // 2 at pow of order
const word MAGIC=484;   // magic constant

// number of max spaces between output values
const int MAX_BLANK=4;

// list of sequential primes numbers
const word PRIME[ORDER*ORDER]={
          7,   11,   13,   17,   19,   23,   29,   31,
         37,   41,   43,   47,   53,   59,   61,   67,   71,   73,   79,
         83,   89,   97,  101,  103,  107,  109,  113,  127,  131,  137,
        139,  149,  151,  157,  163,  167};

// this is a union of bits and the corresponding int value.
// if the program never produce results, probably your compiler manage this
// union in a different way. If we increase 'counter' then some bits may
// changes their value.
union unionBits {
  int counter;
  struct {
    unsigned bit0 :1;
    unsigned bit1 :1;
    unsigned bit2 :1;
    unsigned bit3 :1;
    unsigned bit4 :1;
    unsigned bit5 :1;
  } bit;
};

// note: these defines works ok with even order

#define UP_SEQ {0, 1, 2, 3, 4, 5}
#define DN_SEQ {5, 4, 3, 2, 1, 0}

#define IJ(a) (i==a || j==(ORDER-a-1) || j==a || i==(ORDER-a-1))

#define POS(a) (pos[a]!=a)
#define POSES   POS(0) && POS(1) && POS(2) && POS(3) && \
                POS(4) && POS(5)

#define VALUE(a) (sq.cell[pos[a]][a])
#define VALUES  (VALUE(0)+ VALUE(1)+ VALUE(2)+ VALUE(3)+ VALUE(4)+ \
                 VALUE(5))

#define IF_SWAP if (bits.bit.bit0) swap(sq, i, 0, j, 0); \
                if (bits.bit.bit1) swap(sq, i, 1, j, 1); \
                if (bits.bit.bit2) swap(sq, i, 2, j, 2); \
                if (bits.bit.bit3) swap(sq, i, 3, j, 3); \
                if (bits.bit.bit4) swap(sq, i, 4, j, 4); \
                if (bits.bit.bit5) swap(sq, i, 5, j, 5);

#define SUM_CELL sq.cell[index[bits.bit.bit0]][0]+ \
                 sq.cell[index[bits.bit.bit1]][1]+ \
                 sq.cell[index[bits.bit.bit2]][2]+ \
                 sq.cell[index[bits.bit.bit3]][3]+ \
                 sq.cell[index[bits.bit.bit4]][4]+ \
                 sq.cell[index[bits.bit.bit5]][5]

#define BIT_IJ (bits.bit.bit0 && IJ(0)) || \
               (bits.bit.bit1 && IJ(1)) || \
               (bits.bit.bit2 && IJ(2)) || \
               (bits.bit.bit3 && IJ(3)) || \
               (bits.bit.bit4 && IJ(4)) || \
               (bits.bit.bit5 && IJ(5))

#define cerr_h  \
               "Usage: pms6_7 [options]" << endl \
            << "Options:" << endl \
            << " -m<mh> where mh is the method to use (default d3):" << endl \
            << "    a1:  6R  6C  2D  6R   v1\tdon't give speed result" << endl \
            << "    a2:  6R  6C  2D  6R   v2\tdon't give speed result" << endl \
            << "    a3:  6R  6C  2D  6R   v3\tdon't give speed result" << endl \
            << "    b1:  2R  2D  6R  6C   v1\tdon't give speed result" << endl \
            << "    b2:  2R  2D  6R  6C   v2\tdon't give speed result" << endl \
            << "    b3:  2R  2D  6R  6C   v3\tdon't give speed result" << endl \
            << "    c1:  6R  6C 1D        v1\t4 sq/min in PII 266Mhz " \
            <<                           "(average of 10 squares)" << endl \
            << "    c2:  6R  6C 1D        v2\tdon't give speed result" << endl \
            << "    c3:  6R  6C 1D        v3\t5 sq/min in PII 266Mhz " \
            <<                           "(average of 30 squares)" << endl \
            << "    d1:  6R  6C 1D 1D  6R v1\tdon't give speed result" << endl \
            << "    d2:  6R  6C 1D 1D  6R v2\tdon't give speed result" << endl \
            << "    d3:  6R  6C 1D 1D  6R v3\t3 sq/min in PII 266Mhz " \
            <<                           "(average of 20 squares)" << endl \
            << "    d4:  6R  6C 1D 1D  6R v3\t5 sq/min in PII 266Mhz " \
            <<                           "(average of 20 squares)" << endl \
            << "    (R=row C=column D=diagonal, v version)" << endl \
            << " -n<nu> where nu is the number of squares to generate " \
            <<          "(default 1)" << endl \
            << " -t<nn> where nn is the number of cycle for the methods that " \
            <<                         "allow time out (only c3)" << endl \
            << " -h     the help" << endl;

#define cerr_default "Invalid option: use pms6_7 -h for help" << endl;
