/**
 * @(#)pms4.cpp 2004/01/01
 *
 * @author Tognon Stefano
 * @version 1.00
 *
 * This program automatically generates random prime magic squares of even/odd order.
 * The square is printed in standard output, while the other information are
 * printed in standard error.
 * It divided the square in 4 subsquare and then apply the normal algorithms
 * with MAGIC CONSTANT/2 
 *
 * note: some comments are in the style of Javadoc of Java language!
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307  USA.
 */

#include <iostream.h>
#include <stdlib.h>
#include <sys/timeb.h>
#include macro

#if (DORDER % 2 == 1)
#define ODD
#endif
#if (DORDER % 2 == 0)
#define EVEN
#endif

#if (DORDER % 4 == 1)
#error This order is not valid
#endif

typedef unsigned int word;

// the square representation
struct square {
  int order;                  // actual order of this square (order<=ORDER)
  word magic;                 // actual magic constant of this square (for 'order')  
  word cell[ORDER][ORDER];    // cells elements
  word row[ORDER];            // rows sums
  word col[ORDER];            // columns sums
  word diag1;                 // diag 1 \ sums
  word diag2;                 // diag 2 / sums
};

// square of normal values (1,2,3...)
square normalSquare;

// square of primes values (1,3,5,..)
square primeSquare;

// subsquare
square p1,p2,p3,p4;

// actual time out for timing loop
long int timeOut;

// time out limit of timing loop
long int timeOutLimit=0;

/**
 * Erase the square.
 *
 * @param sq the square to fill with 0
 */
void eraseSquare(square &sq) {
  for (int i=0; i<sq.order; i++) {
    for (int j=0; j<sq.order; j++)
      sq.cell[i][j]=0u;                            // fill the cells element
    sq.row[i]=0u;                                  // fill the rows sum
    sq.col[i]=0u;                                  // fill the columns sum
  }
  sq.diag1=0u;                                     // fill diagonal \
  sq.diag2=0u;                                     // fill diagonal /
}

/**
 * Init the square with random normal numbers
 * It tries to put equals number sum to all the 4 subsquares
 *
 * @param normal the square to
 */
void initRandom(square &sq) {
  int i;  // row index
  int j;  // column index
/*
  for (int n=1; n<=sq.order*sq.order; n+=4) {
    do {
      i=(int)(drand48()*(sq.order/2));             // generate an row index
      j=(int)(drand48()*(sq.order/2));             // generate an column index
    } while (sq.cell[i][j]!=0);                    // is empty?
    sq.cell[i][j]=n;                               // mark with actual value  
    
    do {
      i=(int)(drand48()*(sq.order/2));             // generate an row index
      j=(int)(drand48()*(sq.order/2));             // generate an column index
    } while (sq.cell[i+sq.order/2][j]!=0);         // is empty?    
    sq.cell[i+sq.order/2][j]=n+1;                  // mark with actual value
    
    do {
      i=(int)(drand48()*(sq.order/2));             // generate an row index
      j=(int)(drand48()*(sq.order/2));             // generate an column index
    } while (sq.cell[i][j+sq.order/2]!=0);         // is empty?
    sq.cell[i][j+sq.order/2]=n+2;                  // mark with actual value     
    
    do {
      i=(int)(drand48()*(sq.order/2));             // generate an row index
      j=(int)(drand48()*(sq.order/2));             // generate an column index
    } while (sq.cell[i+sq.order/2][j+sq.order/2]!=0); // is empty?
    sq.cell[i+sq.order/2][j+sq.order/2]=n+3;       // mark with actual value     
  }
*/
  for (int n=1; n<=sq.order*sq.order; n+=8) {
    do {
      i=(int)(drand48()*(sq.order/2));             // generate an row index
      j=(int)(drand48()*(sq.order/2));             // generate an column index
    } while (sq.cell[i][j]!=0);                    // is empty?
    sq.cell[i][j]=n;                               // mark with actual value  
    do {
      i=(int)(drand48()*(sq.order/2));             // generate an row index
      j=(int)(drand48()*(sq.order/2));             // generate an column index
    } while (sq.cell[i][j]!=0);                    // is empty?
    sq.cell[i][j]=n+7;                             // mark with actual value  
    
    do {
      i=(int)(drand48()*(sq.order/2));             // generate an row index
      j=(int)(drand48()*(sq.order/2));             // generate an column index
    } while (sq.cell[i+sq.order/2][j]!=0);         // is empty?    
    sq.cell[i+sq.order/2][j]=n+1;                  // mark with actual value
    do {
      i=(int)(drand48()*(sq.order/2));             // generate an row index
      j=(int)(drand48()*(sq.order/2));             // generate an column index
    } while (sq.cell[i+sq.order/2][j]!=0);         // is empty?    
    sq.cell[i+sq.order/2][j]=n+6;                  // mark with actual value
    
    do {
      i=(int)(drand48()*(sq.order/2));             // generate an row index
      j=(int)(drand48()*(sq.order/2));             // generate an column index
    } while (sq.cell[i][j+sq.order/2]!=0);         // is empty?
    sq.cell[i][j+sq.order/2]=n+2;                  // mark with actual value    
     do {
      i=(int)(drand48()*(sq.order/2));             // generate an row index
      j=(int)(drand48()*(sq.order/2));             // generate an column index
    } while (sq.cell[i][j+sq.order/2]!=0);         // is empty?
    sq.cell[i][j+sq.order/2]=n+5;                  // mark with actual value
    
    do {
      i=(int)(drand48()*(sq.order/2));             // generate an row index
      j=(int)(drand48()*(sq.order/2));             // generate an column index
    } while (sq.cell[i+sq.order/2][j+sq.order/2]!=0); // is empty?
    sq.cell[i+sq.order/2][j+sq.order/2]=n+3;       // mark with actual value    
     do {
      i=(int)(drand48()*(sq.order/2));             // generate an row index
      j=(int)(drand48()*(sq.order/2));             // generate an column index
    } while (sq.cell[i+sq.order/2][j+sq.order/2]!=0); // is empty?
    sq.cell[i+sq.order/2][j+sq.order/2]=n+4;       // mark with actual value  
  }
}

/**
 * Write the square content to standard output (use > to put into a file)
 *
 * @param sq the square to write
 */
void writeSquare(square &sq) {
  word tmp;

  for (int i=0; i<sq.order; i++)  {
    for (int j=0; j<sq.order; j++) {
      tmp=sq.cell[i][j];
      cout << tmp;
      for (int s=1; s<MAX_BLANK; s++)              // make an formatted output
        if ((tmp/=10)==0)
          cout << " ";
    }
    cout << endl;
  }
  cout << endl;
}

/**
 * Transform a normal square to the corresponding prime
 *
 * @param sn the input normal square
 * @param sp the output prime square
 */
void transformToPrime(square &sn, square &sp) {
  for (int i=0; i<sp.order; i++)
    for (int j=0; j<sp.order; j++)
      sp.cell[i][j]=PRIME[sn.cell[i][j]-1];
}

/**
 * Calculate the sum of rows, columns and diagonals of the passed square
 *
 * @param sq the square to sum
 */
void sumSquare(square &sq) {
  sq.diag1=0;                                         // erase previous value
  sq.diag2=0;                                         // erase previous value

  for (int i=0; i<sq.order; i++) {
    sq.row[i]=0;                                      // erase previous value
    sq.col[i]=0;                                      // erase previous value
    sq.diag1+=sq.cell[i][i];                          // sum \ diagonal
    sq.diag2+=sq.cell[i][sq.order-i-1];               // sum / diagonal
  }

  for (int i=0; i<sq.order; i++)                      // row index
    for (int j=0; j<sq.order; j++) {                  // column index
      sq.row[i]+=sq.cell[i][j];                       // sum row
      sq.col[j]+=sq.cell[i][j];                       // sum column
    }
}

/**
 * Calculate actual millisecond time
 *
 * @return the millisecond of actual time
 */
long double actualTime() {
  // actual time
  timeb time;

  ftime(&time);
  return (long double)(time.time*1000+time.millitm);
}

/**
 * Verify that a square is magic.
 *
 * @param sq the square to verify
 * @return true if verify ok
 */
bool verifyMagic(square &sq) {
  int i;
  bool res=true;

  sumSquare(sq);                                   // calculate the sums

  cerr << "Order: "<<sq.order<<endl;
  cerr << "Magic: " << sq.magic << endl;

  cerr << "Verify that square is magic:" << endl
       << "rows\t";
  for (int i=0; i<sq.order; i++) {
    if (sq.row[i]==sq.magic) cerr << "@";
    else {
      cerr << "0";
      res=false;
    }
  }

  cerr  << endl << "columns\t";
  for (int i=0; i<sq.order; i++) {
    if (sq.col[i]==sq.magic) cerr << "@";
    else {
      cerr << "0";
      res=false;
    }
  }

  cerr << endl << "diag \\:\t";
  if (sq.diag1==sq.magic) cerr << "@";
  else {
    cerr << "0";
    res=false;
  }

  cerr << endl << "diag /:\t";
  if (sq.diag2==sq.magic) cerr << "@";
  else {
    cerr << "0";
    res=false;
  }
  cerr << endl;
  return res;
}

/**
 * Rotate the square so rows become columns.
 * Note: this rotate and than invert all value at sx to dx, so we don't need
 * another temp square.
 *
 * @param sq the square
 */
void rotateSquare(square &sq) {
  word tmp;

  for (int i=0; i<sq.order-1; i++) {
    for (int j=i+1; j<sq.order; j++) {
      tmp=sq.cell[i][j];
      sq.cell[i][j]=sq.cell[j][i];
      sq.cell[j][i]=tmp;
    }
  }
}

/**
 * Swap two cell in the square
 *
 * @param sq the square where do the swap
 * @param row1 the first row
 * @param col1 the first column
 * @param row2 the second row
 * @param col2 the second column
 */
inline void swap(square &sq, int row1, int col1, int row2, int col2) {
  word tmp=sq.cell[row1][col1];
  sq.cell[row1][col1]=sq.cell[row2][col2];
  sq.cell[row2][col2]=tmp;
}

/**
 * Swap elements or rows j and i for making row i to the magic state.
 *
 * @param sq the square where working
 * @param i the row to make magic
 * @param j the row where choose values
 * @return true if row becomes magic
 */
bool swapping1(square &sq, int i, int j) {

  // this rapresent the combination of elements to choose from rows i and j
  // when we increment counter, the bits changes, so all the combination are
  // given.

  unionBits bits;

  int index[2]={i, j};                             // index for row i and j

  // try all the swap combination for making row i magic
  for (bits.counter=0; bits.counter<COMB; bits.counter++) {
    if (SUM_CELL==sq.magic) {                      // is magic?
      IF_SWAP;                                     // ok, make the swaps
                                                   // that are needed
      return true;                                 // row i is now magic
    }
  }
  return false;                                    // row i is not magic
}

/**
 * Swap elements or rows j and i for making row i to the magic state.
 * Note: we don't change diagonals state.
 *
 * @param sq the square where working
 * @param i the row to make magic
 * @param j the row where choose values
 * @return true is row becomes magic
 */
bool swapping2(square &sq, int i, int j) {

  // this rapresent the combination of elements to choose from rows i and j
  // when we increment counter, the bits changes, so all the combination are
  // given.

  unionBits bits;

  int index[2]={i, j};                             // index for row i and j

  // try all the swap combination for making row i magic
  for (bits.counter=0; bits.counter<COMB; bits.counter++) {
    if (SUM_CELL==sq.magic) {                      // is magic?
      if (BIT_IJ)
       continue;  // don't change elements in the diagonals

      IF_SWAP;                                     // ok, make the swaps
                                                   // that are needed
      return true;                                 // row i is now magic
    }
  }
  return false;                                    // row i is not magic
}

/**
 * Make intelligent swaps for making all rows magic
 * One rows takes value from the next until the last row.
 *
 * @param sq the square to make rows magic
 * @return true if all rows become magic
 */
bool intellSwap1(square &sq) {
  bool ok;
  for (int i=0; i<sq.order-1; i++) {
    ok=false;
    for (int j=i+1; j<sq.order; j++) {
      if (swapping1(sq, i, j)) {                   // swap i with all the other
        ok=true;                                   // row ok
        break;                                     // go to other rows
      }
    }
    if (!ok) return false;
  }
  return true;
}

/**
 * Make intelligent swaps for making all row magic
 *
 * @param sq the square to make rows magic
 * @return true if all rows become magic
 */
bool intellSwap2(square &sq) {
  bool ok;
  for (int i=0; i<sq.order-1; i++) {
    ok=false;
    for (int j=i+1; j<sq.order; j++) {            // swap i with all the other
      if (swapping2(sq, i, j)) {
        ok=true;                                  // row ok
        break;                                    // go to other rows
      }
    }
    if (!ok) return false;                        // choose another square
  }
  return true;
}

/**
 * Make intelligent swaps for making all rows magic
 * One rows takes value from the next until the last row, but with different
 * sign from the magic state.
 *
 * @param sq the square to make rows magic
 * @return true if all rows become magic
 */
bool intellSwap3(square &sq) {
  bool ok;
  bool sign;

  sumSquare(sq);
  for (int i=0; i<sq.order-1; i++) {
    ok=false;
    sign=(sq.row[i]>sq.magic);                     // get actual sign
    for (int j=i+1; j<sq.order; j++) {
      if (((sq.row[j]>sq.magic))!=sign) {          // are different?
        if (swapping1(sq, i, j)) {                 // swap i with all the other
          sq.row[j]=0;
          for (int k=0; k<sq.order; k++)           // update sum
            sq.row[j]+=sq.cell[j][k];
          ok=true;                                 // row ok
          break;                                   // go to other rows
        }
      }
    }
    if (!ok) return false;
  }
  return true;
}

/**
 * Make intelligent swaps for making all rows magic
 * One rows takes value from the next until the last row, but with different
 * sign from the magic state and don't changing diagonals value.
 *
 * @param sq the square to make rows magic
 * @return true if all rows become magic
 */
bool intellSwap4(square &sq) {
  bool ok;
  bool sign;

  sumSquare(sq);
  for (int i=0; i<sq.order-1; i++) {
    ok=false;
    sign=(sq.row[i]>sq.magic);                     // get actual sign
    for (int j=i+1; j<sq.order; j++) {
      if (((sq.row[j]>sq.magic))!=sign) {          // are different?
        if (swapping2(sq, i, j)) {                 // swap i with all the other
          sq.row[j]=0;
          for (int k=0; k<sq.order; k++)         // update sum
            sq.row[j]+=sq.cell[j][k];
          ok=true;                                 // row ok
          break;                                   // go to other rows
        }
      }
    }
    if (!ok) return false;
  }

  return true;
}

/**
 * Permutate value for making diagonal \ magic without changing rows and
 * columns magic state.
 *
 * @param pos contains actual position in a column of choosed values
 * @param w an index for recursion
 * @param sq the square
 * @return true if ok
 */
bool permDiag1(int *pos, int w, square &sq) {
  int tmp;
  int n;

  if (w>=sq.order-1) {
      if (VALUES==sq.magic) {                      // row \ is magic

        square tmp;                                // tmp square
        tmp.order=sq.order;
        tmp.magic=sq.magic;

        for (int i=0; i<sq.order; i++) {
          for (int j=0; j<sq.order; j++) {        // changes the row position
            tmp.cell[i][j]=sq.cell[pos[i]][j];
          }
        }
        sq=tmp;                                   // return the new square
        return true;
      }
  } else {
      for (n=w; n<sq.order; n++) {                // calc. all the permutation
        tmp=pos[n];
        pos[n]=pos[w];
        pos[w]=tmp;
        if (permDiag1(pos, (w+1), sq))
          return true;
        tmp=pos[n];
        pos[n]=pos[w];
        pos[w]=tmp;
      }
    }
  return false;
}

/**
 * Permutate value for making diagonal \ magic until row / become magic too.
 *
 * @param pos contains actual position in a column of choosed values
 * @param w an index for recursion
 * @param sq the square
 * @return true if ok
 */
bool permDiag2(int *pos, int w, square &sq) {
  int tmp;
  int n;

  if (w>=sq.order-1) {
      if (VALUES==sq.magic) {                      // row \ is magic

        square tmp;                                // tmp square
        tmp.order=sq.order;
        tmp.magic=sq.magic;
        word sum=0;                                // sum of diagonal values

        for (int i=0; i<sq.order; i++) {
          for (int j=0; j<sq.order; j++) {         // changes the row position
            tmp.cell[i][j]=sq.cell[pos[i]][j];
          }
          sum+=tmp.cell[i][sq.order-1-i];
        }
        if (sum==sq.magic) {                       // if square all magic?
          sq=tmp;                                  // return the new square
          return true;
        }
      }
  } else {
      for (n=w; n<sq.order; n++) {                 // calc. all the permutation
        tmp=pos[n];
        pos[n]=pos[w];
        pos[w]=tmp;
        if (permDiag2(pos, (w+1), sq))
          return true;
        tmp=pos[n];
        pos[n]=pos[w];
        pos[w]=tmp;
      }
    }
  return false;
}

/**
 * Permutate value for making diagonal \ magic until row / become magic too, or
 * there's time out.
 *
 * @param pos contains actual position in a column of choosed values
 * @param w an index for recursion
 * @param sq the square
 * @return true if ok or time out
 */
bool permDiag3(int *pos, int w, square &sq) {
  int tmp;
  int n;

  if (w>=sq.order-1) {
      timeOut++;

      if (timeOut>timeOutLimit)
        return true;                               // return true but not -1
      if (VALUES==sq.magic) {                      // row \ is magic

        square tmp;                                // tmp square
        tmp.order=sq.order;
        tmp.magic=sq.magic;
        word sum=0;                                // sum of diagonal values

        for (int i=0; i<sq.order; i++) {
          for (int j=0; j<sq.order; j++) {         // changes the row position
            tmp.cell[i][j]=sq.cell[pos[i]][j];
          }
          sum+=tmp.cell[i][sq.order-1-i];
        }
        if (sum==sq.magic) {                       // if square all magic?
          cerr << "Time out: " << timeOut << endl;
          sq=tmp;                                  // return the new square
          timeOut=-1;
          return true;                             // return true with -1
        }
      }
  } else {
      for (n=w; n<sq.order; n++) {                 // calc. all the permutation
        tmp=pos[n];
        pos[n]=pos[w];
        pos[w]=tmp;
        if (permDiag3(pos, (w+1), sq))
          return true;
        tmp=pos[n];
        pos[n]=pos[w];
        pos[w]=tmp;
      }
    }
  return false;
}

/**
 * Permutate value for making diagonal / magic without changing \ diagonal
 *
 * @param pos contains actual position in a column of choosed values
 * @param w an index for recursion
 * @param sq the square
 * @return true if ok
 */
bool permDiag4(int *pos, int w, square &sq) {
  int tmp;
  int n;

  if (w>=sq.order-1) {
      if (VALUES==sq.magic) {                         // row / is magic
          // don't change \ diagonal
        if (POSES) {
          for (int i=0; i<sq.order; i++) {
            swap(sq, pos[i], i, sq.order-1-i, i);     // change only cells
          }
          return true;
        }
      }
  } else {
      for (n=w; n<sq.order; n++) {                    // calc. all the permutation
        tmp=pos[n];
        pos[n]=pos[w];
        pos[w]=tmp;
        if (permDiag4(pos, (w+1), sq))
          return true;
        tmp=pos[n];
        pos[n]=pos[w];
        pos[w]=tmp;
      }
    }
  return false;
}

/**
 * Makes the two diagonals magic with method 1
 * Algorithm for even :
 * copy the first row to the diagonal \ and the last row to diagonal /
 * So, this works only ok if rows one and last are magic.
 * if columns were magic, this don't change the magic state, while if rows were
 * magic, they become not magic.
 *
 * Algorithm for odd:
 * copy the half row to the diagonal \ and the half column to diagonal /
 * This changes the magic state of rows and columns.
 *
 * @param sq the square
 */
void diagonal1(square &sq) {
#ifdef EVEN
  for (int i=1; i<sq.order-1; i++) {
    swap(sq, 0, i, i, i);
    swap(sq, sq.order-1, i, sq.order-1-i, i);
  }
  swap(sq, 0, sq.order-1, sq.order-1, sq.order-1);          // swap the UpDx with DnSx
#else
  for (int i=0; i<sq.order; i++) {
    swap(sq, sq.order/2, i, i, i);
    swap(sq, i, sq.order/2, sq.order-1-i, i);
  }
#endif  
}

/**
 * Makes the diagonal \ magic and see if diagonal / become magic, or repeat
 * forever.
 *
 * @param sq the square
 * @return true if diagonals are magics
 */
bool diagonal2(square &sq) {
  // position in a column of element to put into the diagonal
  int position[ORDER]=UP_SEQ;

  return permDiag2(position, 0, sq);
}

/**
 * Makes the diagonal \ magic and see if diagonal / become magic, or repeat
 * for a max time.
 *
 * @param sq the square
 * @return true if build ok, false if time out
 */
bool diagonal3(square &sq) {
  // position in a column of element to put into the diagonal
  int position[ORDER]=UP_SEQ;

  timeOut=0;
  if (permDiag3(position, 0, sq))
    if (timeOut==-1)
      return true;
  return false;
}

/**
 * Makes the diagonal \ magic without changing rows and columns state, then
 * makes the other / changing columns and rows.
 *
 * @param sq the square
 * @return true if diagonals magics
 */
bool diagonal4(square &sq) {
  // position in a column of element to put into the diagonal
  int position1[ORDER]=UP_SEQ;
  int position2[ORDER]=DN_SEQ;

  if (!permDiag1(position1, 0, sq)) return false;
  if (!permDiag4(position2, 0, sq)) return false;
  return true;
}

/**
 * Apply method A1 of magic solution to the passed prime square.
 * Algorithm:
 * 1: make all rows magic
 * 2: rotate the square
 * 3: make rows magic (that was columns in point 1)
 * 4: make diagonals magic using two magic rows
 * 5: remake all rows magic without change diagonals state
 * 6: rotate the square
 * 7: remake all rows (columns in point 5) magic without change diagonals state
 * version 1
 *
 * @param sq the square with prime number
 * @return true if sucessful building
 */
bool magicA1(square &sq) {
  if (!intellSwap1(sq))
    return false;                                  // point 1
  rotateSquare(sq);                                // point 2
  if (!intellSwap1(sq))
    return false;                                  // point 3
  diagonal1(sq);                                   // point 4
  if (!intellSwap2(sq))                            // point 5 
    return false;
#ifdef ODD    
  rotateSquare(sq);                                // point 6
  if (!intellSwap2(sq))                            // point 7
    return false;   
#endif    
  return true;
}

/**
 * Apply method A2 of magic solution to the passed prime square.
 * Algorithm:
 * 1: make all rows magic
 * 2: rotate the square
 * 3: make rows magic (that was columns in point 1)
 * 4: make diagonals magic using two magic rows
 * 5: remake all rows magic without change diagonals state
 * 6: rotate the square
 * 7: remake all rows (columns in point 5) magic without change diagonals state
 * version 2
 *
 * @param sq the square with prime number
 * @return true if sucessful building
 */
bool magicA2(square &sq) {
  if (!intellSwap3(sq))
    return false;                                  // point 1
  rotateSquare(sq);                                // point 2
  if (!intellSwap3(sq))
    return false;                                  // point 3
  diagonal1(sq);                                   // point 4
  if (!intellSwap4(sq))                            // point 5 
    return false;
#ifdef ODD    
  rotateSquare(sq);                                // point 6
  if (!intellSwap4(sq))                            // point 7
    return false;
#endif        
  return true;
}

/**
 * Apply method A3 of magic solution to the passed prime square.
 * Algorithm:
 * 1: make all rows magic
 * 2: rotate the square
 * 3: make rows magic (that was columns in point 1)
 * 4: make diagonals magic using two magic rows
 * 5: remake all rows magic without change diagonals state
 * 6: rotate the square
 * 7: remake all rows (columns in point 5) magic without change diagonals state
 * version 3
 *
 * @param sq the square with prime number
 * @return true if sucessful building
 */
bool magicA3(square &sq) {
  if (!intellSwap3(sq))
    if (!intellSwap1(sq))
      return false;                                // point 1
  rotateSquare(sq);                                // point 2
  if (!intellSwap3(sq))
    if (!intellSwap1(sq))
      return false;                                // point 3
  diagonal1(sq);                                   // point 4
  if (!intellSwap4(sq))                            // point 5 
    if (!intellSwap2(sq))
      return false;
#ifdef ODD      
  rotateSquare(sq);                                // point 6
  if (!intellSwap4(sq))                            // point 7
    if (!intellSwap2(sq))
      return false;      
#endif       
  return true;
}

/**
 * Apply method B1 of magic solution to the passed prime square.
 * Algorithm:
 * 1: make 2 rows magic
 * 2: make diagonals magic using two magic rows
 * 3: remake all rows magic without change diagonals state
 * 4: rotate the square
 * 5: remake all rows (columns in point 3) magic without change diagonals state
 * version 1
 *
 * @param sq the square with prime number
 * @return true if sucessful building
 */
bool magicB1(square &sq) {
#ifdef EVEN
  if (!swapping1(sq, 0, 1))                        // point 1
    return false;
  if (!swapping1(sq, sq.order-1, sq.order-2))
    return false;
#else    
  if (!swapping1(sq, sq.order/2, 1))               // point 1
    return false;
  rotateSquare(sq);
  if (!swapping1(sq, sq.order/2, 1))
    return false;    
#endif    
    
  diagonal1(sq);                                   // point 2
  if (!intellSwap2(sq))
    return false;                                  // point 3
  rotateSquare(sq);                                // point 4
  if (!intellSwap2(sq))
    return false;                                  // point 5
  return true;
}

/**
 * Apply method B2 of magic solution to the passed prime square.
 * Algorithm:
 * 1: make 2 rows magic
 * 2: make diagonals magic using two magic rows
 * 3: remake all rows magic without change diagonals state
 * 4: rotate the square
 * 5: remake all rows (columns in point 3) magic without change diagonals state
 * version 2
 *
 * @param sq the square with prime number
 * @return true if sucessful building
 */
bool magicB2(square &sq) {
#ifdef EVEN
  if (!swapping1(sq, 0, 1))                        // point 1
    return false;
  if (!swapping1(sq, sq.order-1, sq.order-2))
    return false;
#else
  if (!swapping1(sq, sq.order/2, 1))                  // point 1
    return false;
  rotateSquare(sq);
  if (!swapping1(sq, sq.order/2, 1))
    return false;
#endif    
  diagonal1(sq);                                   // point 2
  if (!intellSwap4(sq)) return false;              // point 3
  rotateSquare(sq);                                // point 4
  if (!intellSwap4(sq)) return false;              // point 5
  return true;
}

/**
 * Apply method B3 of magic solution to the passed prime square.
 * Algorithm:
 * 1: make 2 rows magic
 * 2: make diagonals magic using two magic rows
 * 3: remake all rows magic without change diagonals state
 * 4: rotate the square
 * 5: remake all rows (columns in point 3) magic without change diagonals state
 * version 3
 *
 * @param sq the square with prime number
 * @return true if sucessful building
 */
bool magicB3(square &sq) {
#ifdef EVEN
  if (!swapping1(sq, 0, 1))                        // point 1
    return false;
  if (!swapping1(sq, sq.order-1, sq.order-2))
    return false;
#else
  if (!swapping1(sq, sq.order/2, 1))               // point 1
    return false;
  rotateSquare(sq);
  if (!swapping1(sq, sq.order/2, 1))
    return false;
#endif    
    
  diagonal1(sq);                                   // point 2
  if (!intellSwap4(sq))
    if (!intellSwap2(sq))
      return false;                                // point 3
  rotateSquare(sq);                                // point 4
  if (!intellSwap4(sq))
    if (!intellSwap2(sq))
      return false;                                // point 5
  return true;
}

/**
 * Apply method C1 of magic solution to the passed prime square.
 * Algorithm:
 * 1: make all rows magic
 * 2: rotate the square
 * 3: make rows magic (that was columns in point 1)
 * 4: make one diagonals magic using permutation (and pray for the other)
 * version 1
 *
 * @param sq the square with prime number
 * @return true if sucessful building
 */
bool magicC1(square &sq) {
  if (!intellSwap1(sq))
    return false;                                  // point 1
  rotateSquare(sq);                                // point 2
  if (!intellSwap1(sq))
    return false;                                  // point 3
  return diagonal2(sq);                            // point 4
}

/**
 * Apply method C2 of magic solution to the passed prime square.
 * Algorithm:
 * 1: make all rows magic
 * 2: rotate the square
 * 3: make rows magic (that was columns in point 1)
 * 4: make one diagonals magic using permutation (and pray for the other)
 * version 2
 *
 * @param sq the square with prime number
 * @return true if sucessful building
 */
bool magicC2(square &sq) {
  if (!intellSwap3(sq))
    return false;                                  // point 1
  rotateSquare(sq);                                // point 2
  if (!intellSwap3(sq))
    return false;                                  // point 3
  return diagonal2(sq);                            // point 4
}

/**
 * Apply method C3 of magic solution to the passed prime square.
 * Algorithm:
 * 1: make all rows magic
 * 2: rotate the square
 * 3: make rows magic (that was columns in point 1)
 * 4: make one diagonals magic using permutation (and pray for the other)
 * version 3
 *
 * @param sq the square with prime number
 * @return true if sucessful building
 */
bool magicC3(square &sq) {
  if (!intellSwap3(sq))
    if (!intellSwap1(sq))
      return false;                                // point 1
  rotateSquare(sq);                                // point 2
  if (!intellSwap3(sq))
    if (!intellSwap1(sq))
      return false;                                // point 3
  if (timeOutLimit!=0) {                           // point 4
    if (!diagonal3(sq))                            // use time-out
      return false;
  } else return diagonal2(sq);                     // don't use timeout
  return true;
}

/**
 * Apply method D1 of magic solution to the passed prime square.
 * Algorithm:
 * 1: make all rows magic
 * 2: rotate the square
 * 3: make rows magic (that was columns in point 1)
 * 4: make diagonals magic using two algoritms
 * 5: remake all rows magic without change diagonals state
 * 6: rotate the square
 * 7: remake all rows (columns in point 5) magic without change diagonals state
 * version 1
 *
 * @param sq the square with prime number
 * @return true if sucessful building
 */
bool magicD1(square &sq) {
  if (!intellSwap1(sq))
    return false;                                  // point 1
  rotateSquare(sq);                                // point 2
  if (!intellSwap1(sq))
    return false;                                  // point 3
  if (!diagonal4(sq)) return false;                // point 4
  if (!intellSwap2(sq))                            // point 5 (6, 7 not needed)
    return false;
  return true;
}

/**
 * Apply method D2 of magic solution to the passed prime square.
 * Algorithm:
 * 1: make all rows magic
 * 2: rotate the square
 * 3: make rows magic (that was columns in point 1)
 * 4: make diagonals magic using two algoritms
 * 5: remake all rows magic without change diagonals state
 * 6: rotate the square
 * 7: remake all rows (columns in point 5) magic without change diagonals state
 * version 2
 *
 * @param sq the square with prime number
 * @return true if sucessful building
 */
bool magicD2(square &sq) {
  if (!intellSwap3(sq))
    return false;                                  // point 1
  rotateSquare(sq);                                // point 2
  if (!intellSwap3(sq))
    return false;                                  // point 3
  if (!diagonal4(sq)) return false;                // point 4
  if (!intellSwap4(sq))                            // point 5 (6, 7 not needed)
    return false;
  return true;
}

/**
 * Apply method D3 of magic solution to the passed prime square.
 * Algorithm:
 * 1: make all rows magic
 * 2: rotate the square
 * 3: make rows magic (that was columns in point 1)
 * 4: make diagonals magic using two algoritms
 * 5: remake all rows magic without change diagonals state
 * 6: rotate the square
 * 7: remake all rows (columns in point 5) magic without change diagonals state
 * version 3
 *
 * @param sq the square with prime number
 * @return true if sucessful building
 */
bool magicD3(square &sq) {
  if (!intellSwap3(sq))
    if (!intellSwap1(sq))
      return false;                                // point 1  
  rotateSquare(sq);                                // point 2  
  if (!intellSwap3(sq))
    if (!intellSwap1(sq))
      return false;                                // point 3
  if (!diagonal4(sq)) return false;                // point 4
  if (!intellSwap4(sq))                            // point 5 (6, 7 not needed)
    if (!intellSwap2(sq))
      return false;
  return true;
}

/**
 * Apply method D4 of magic solution to the passed prime square.
 * Algorithm:
 * 1: make all rows magic
 * 2: rotate the square
 * 3: make rows magic (that was columns in point 1)
 * 4: make diagonals magic using two algoritms
 * 5: remake all rows magic without change diagonals state
 * 6: rotate the square
 * 7: remake all rows (columns in point 5) magic without change diagonals state
 * version 4
 *
 * @param sq the square with prime number
 * @return true if sucessful building
 */
bool magicD4(square &sq) {
  if (!intellSwap3(sq))
    if (!intellSwap1(sq)) {
      rotateSquare(sq);                            // try with columns
      if (!intellSwap3(sq))
        if (!intellSwap1(sq)) {
          rotateSquare(sq);
          if (!intellSwap3(sq))
           if (!intellSwap1(sq))
            return false;                          // point 1
        }
    }
  rotateSquare(sq);                                // point 2
  if (!intellSwap3(sq))
    if (!intellSwap1(sq))
      return false;                                // point 3
  if (!diagonal4(sq)) return false;                // point 4
  if (!intellSwap4(sq))                            // point 5 (6, 7 not needed)
    if (!intellSwap2(sq))
      return false;
  return true;
}


/**
 * Divide a square in 4 subsquare
 *
 * @param sq the big square 
 * @param p1 the subsquare 1 
 * @param p2 the subsquare 2 
 * @param p3 the subsquare 3
 * @param p4 the subsquare 4
 */
void flipSquare(square &sq, square &p1, square &p2, square &p3, square &p4) {
  p1.order=p2.order=p3.order=p4.order=sq.order/2;
  p1.magic=p2.magic=p3.magic=p4.magic=sq.magic/2;
  
  for (int i=0; i<sq.order/2; i++) {
    for (int j=0; j<sq.order/2; j++) {
      p1.cell[i][j]=sq.cell[i][j];
      p2.cell[i][j]=sq.cell[i+sq.order/2][j];
      p3.cell[i][j]=sq.cell[i][j+sq.order/2];
      p4.cell[i][j]=sq.cell[i+sq.order/2][j+sq.order/2];
    }
  }
}

/**
 * Sum 4 subsquare in one square
 *
 * @param sq the big square 
 * @param p1 the subsquare 1 
 * @param p2 the subsquare 2 
 * @param p3 the subsquare 3
 * @param p4 the subsquare 4
 */
void unflipSquare(square &sq, square &p1, square &p2, square &p3, square &p4) {
  for (int i=0; i<sq.order/2; i++) {
    for (int j=0; j<sq.order/2; j++) {
      sq.cell[i][j]=p1.cell[i][j];
      sq.cell[i+sq.order/2][j]=p2.cell[i][j];
      sq.cell[i][j+sq.order/2]=p3.cell[i][j];
      sq.cell[i+sq.order/2][j+sq.order/2]=p4.cell[i][j];
    }
  }
}

int main(int argc, char *argv[]) {
  // result of magic building
  bool result;

  // millisecond at start time
  long double startTime;

  // millisecond at end time
  long double endTime;

  int type=43;

  // number of square to generate
  int number=1;

  if (argc>=2) {
    for (int j=1; j<argc; j++) {
      if (argv[j][0]== '-') {
        switch (argv[j][1]) {
          case 'm':
            switch (argv[j][2]) {
              case 'a':
                switch (argv[j][3]) {
                  case '1':
                    cerr << "Method A1:" << endl;
                    type=11;
                    break;
                  case '2':
                    cerr << "Method A2:" << endl;
                    type=12;
                    break;
                  case '3':
                    cerr << "Method A3:" << endl;
                    type=13;
                    break;
                }
                break;
              case 'b':
                switch (argv[j][3]) {
                  case '1':
                    cerr << "Method B1:" << endl;
                    type=21;
                    break;
                  case '2':
                    cerr << "Method B2:" << endl;
                    type=22;
                    break;
                  case '3':
                    cerr << "Method B3:" << endl;
                    type=23;
                    break;
                }
                break;
              case 'c':
                switch (argv[j][3]) {
                  case '1':
                    cerr << "Method C1:" << endl;
                    type=31;
                    break;
                  case '2':
                    cerr << "Method C2:" << endl;
                    type=32;
                    break;
                  case '3':
                    cerr << "Method C3:" << endl;
                    type=33;
                     break;
                }
                break;
              case 'd':
                switch (argv[j][3]) {
                  case '1':
                    cerr << "Method D1:" << endl;
                    type=41;
                    break;
                  case '2':
                    cerr << "Method D2:" << endl;
                    type=42;
                    break;
                  case '3':
                    cerr << "Method D3:" << endl;
                    type=43;
                     break;
                  case '4':
                    cerr << "Method D4:" << endl;
                    type=44;
                    break;
                }
                break;
            }
            break;
          case 'n':
            number=atoi(argv[j]+2);
            if (number==0) number=1;
            break;
          case 't':
            timeOutLimit=atoi(argv[j]+2);
            if (timeOutLimit<0) timeOutLimit=0;
            cerr << "Time out of " << timeOutLimit << " if allowed" << endl;
            cerr << "0 means not use time out" << endl;
            break;
          case 'h':
            cerr << cerr_h;
            return 0;
          default:
            cerr << cerr_default;
            return 1;
        }
      }
    }
  }

  startTime=actualTime();
  srand48((long int)startTime);
  for (int i=1; i<=number; i++) {
    cerr << "square number " << i << endl;
    do {
      // init order/constant values
      normalSquare.order=ORDER;
      primeSquare.order=ORDER;
      normalSquare.magic=MAGIC;
      primeSquare.magic=MAGIC;    
    
      eraseSquare(normalSquare);
      eraseSquare(primeSquare);

      initRandom(normalSquare);
      transformToPrime(normalSquare, primeSquare);

      flipSquare(primeSquare, p1, p2, p3, p4);   // build 4 sub square      
      //writeSquare(primeSquare);
      //writeSquare(p1);
      //writeSquare(p2);
      //writeSquare(p3);
      //writeSquare(p4);
      
      //verifyMagic(primeSquare);
      //verifyMagic(p1);
      //verifyMagic(p2);
      //verifyMagic(p3);
      //verifyMagic(p4);
      //magicD4(p1);
      //verifyMagic(p1);
      //writeSquare(p1);
      
      switch (type) {
        case 11:
          //cerr << "A1" << endl;
          result=magicA1(p1) && magicA1(p2) && magicA1(p3) && magicA1(p4);
          break;
        case 12:
          //cerr << "A2" << endl;
          result=magicA2(p1) && magicA2(p2) && magicA2(p3) && magicA2(p4);
          break;
        case 13:
          //cerr << "A3" << endl;
          result=magicA3(p1) && magicA3(p2) && magicA3(p3) && magicA3(p4);
          break;
        case 21:
          //cerr << "B1" << endl;
          result=magicB1(p1) && magicB1(p2) && magicB1(p3) && magicB1(p4);
          break;
        case 22:
          //cerr << "B2" << endl;
          result=magicB2(p1) && magicB2(p2) && magicB2(p3) && magicB2(p4);
          break;
        case 23:
          //cerr << "B3" << endl;
          result=magicB3(p1) && magicB3(p2) && magicB3(p3) && magicB3(p4);
          break;
        case 31:
          //cerr << "C1" << endl;
          result=magicC1(p1) && magicC1(p2) && magicC1(p3) && magicC1(p4);
          break;
        case 32:
          //cerr << "C2" << endl;
          result=magicC2(p1) && magicC2(p2) && magicC2(p3) && magicC2(p4);
          break;
        case 33:
          //cerr << "C3" << endl;
          result=magicC3(p1) && magicC3(p2) && magicC3(p3) && magicC3(p4);
          break;
        case 41:
          //cerr << "D1" << endl;
          result=magicD1(p1) && magicD1(p2) && magicD1(p3) && magicD1(p4);
          break;
        case 42:
          //cerr << "D2" << endl;
          result=magicD2(p1) && magicD2(p2) && magicD2(p3) && magicD2(p4);
          break;
        case 43:
          //cerr << "D3" << endl;
          ///result=magicD3(p1) && magicD3(p2) && magicD3(p3) && magicD3(p4);
          result=magicD3(p1);
          if (!result) break;
          cerr << " 1\n";

          result=magicD3(p2);
          if (!result) break;
          cerr << " 2\n";

          result=magicD3(p3);
          if (!result) break;
          cerr << " 3\n";

          result=magicD3(p2);
          if (!result) break;
          cerr << " 4\n";



          break;
        case 44:
          //cerr << "D4" << endl;
          result=magicD4(p1) && magicD4(p2) && magicD4(p3) && magicD4(p4);
          break;
      }
    } while(!result);
      verifyMagic(p1);
      verifyMagic(p2);
      verifyMagic(p3);
      verifyMagic(p4);
    unflipSquare(primeSquare, p1, p2, p3, p4);

    writeSquare(primeSquare);

    if (!verifyMagic(primeSquare))
      cout << endl << "ERROR in verify: there's an error not view before now"
           << endl;
  }

  // calculate and show statistic information
  endTime=actualTime();

  cerr << endl;
  cerr << "Total Working Time:\t\t" << ((endTime-startTime)/1000)
       << "\tseconds" << endl;
  cerr << "So in a second I can build:\t" << (1000/(endTime-startTime)*number)
       << "\tsquares" << endl;
  cerr << "or in a minute I can build:\t" << (1000*60/(endTime-startTime)*number)
       << "\tsquares" << endl;
  cerr << "in a hour I can build:\t\t" << (1000*60*60/(endTime-startTime)*number)
       << "\tsquares" << endl;
  cerr << "in a day I can build:\t\t"
       << (1000*60*60/(endTime-startTime)*24*number)
       << "\tsquares" << endl;
  cerr << "in a year I can build:\t\t"
       << (1000*60*60/(endTime-startTime)*24*365*number)
       << "\tsquares" << endl;

  return 0;
}
