/**
 * @(#)pmsall.cpp 2000/03/22
 *
 * @author Tognon Stefano
 * @version 1.00
 *
 * This program automatically search all combination for prime magic square
 * of low order.
 * The square is printed in standard output, while the other information are
 * printed in standard error.
 *
 * note: some comments are in the style of Javadoc of Java language!
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307  USA.
 */

#include <iostream.h>
#include <stdlib.h>
#include <sys/timeb.h>
#include macro

typedef unsigned int word;

// the square representation
struct square {
  word cell[ORDER][ORDER];    // cells elements
  word row[ORDER];            // rows sums
  word col[ORDER];            // columns sums
  word diag1;                 // diag 1 \ sums
  word diag2;                 // diag 2 / sums
};

// square of primes values (1,3,5,..)
square primeSquare;

/**
 * Erase the square.
 *
 * @param sq the square to fill with 0
 */
void eraseSquare(square &sq) {
  for (int i=0; i<ORDER; i++) {
    for (int j=0; j<ORDER; j++)
      sq.cell[i][j]=0u;                            // fill the cells element
    sq.row[i]=0u;                                  // fill the rows sum
    sq.col[i]=0u;                                  // fill the columns sum
  }
  sq.diag1=0u;                                     // fill diagonal \
  sq.diag2=0u;                                     // fill diagonal /
}

/**
 * Write the square content to standard output (use > to put into a file)
 *
 * @param sq the square to write
 */
void writeSquare(square &sq) {
  word tmp;

  for (int i=0; i<ORDER; i++)  {
    for (int j=0; j<ORDER; j++) {
      tmp=sq.cell[i][j];
      cout << tmp;
      for (int s=1; s<MAX_BLANK; s++)              // make an formatted output
        if ((tmp/=10)==0)
          cout << " ";
    }
    cout << endl;
  }
  cout << endl;
}

/**
 * Calculate the sum of rows, columns and diagonals of the passed square
 *
 * @param sq the square to sum
 */
void sumSquare(square &sq) {
  sq.diag1=0;                                         // erase previous value
  sq.diag2=0;                                         // erase previous value

  for (int i=0; i<ORDER; i++) {
    sq.row[i]=0;                                      // erase previous value
    sq.col[i]=0;                                      // erase previous value
    sq.diag1+=sq.cell[i][i];                          // sum \ diagonal
    sq.diag2+=sq.cell[i][ORDER-i-1];                  // sum / diagonal
  }

  for (int i=0; i<ORDER; i++)                         // row index
    for (int j=0; j<ORDER; j++) {                     // column index
      sq.row[i]+=sq.cell[i][j];                       // sum row
      sq.col[j]+=sq.cell[i][j];                       // sum column
    }
}

/**
 * Calculate actual millisecond time
 *
 * @return the millisecond of actual time
 */
long double actualTime() {
  // actual time
  timeb time;

  ftime(&time);
  return (long double)(time.time*1000+time.millitm);
}

/**
 * Verify that a square is magic.
 *
 * @param sq the square to verify
 * @return true if verify ok
 */
bool verifyMagic(square &sq) {
  int i;
  bool res=true;

  sumSquare(sq);                                   // calculate the sums

  cerr << "Verify that square is magic:" << endl
       << "rows\t";
  for (int i=0; i<ORDER; i++) {
    if (sq.row[i]==MAGIC) cerr << "@";
    else {
      cerr << "0";
      res=false;
    }
  }

  cerr  << endl << "columns\t";
  for (int i=0; i<ORDER; i++) {
    if (sq.col[i]==MAGIC) cerr << "@";
    else {
      cerr << "0";
      res=false;
    }
  }

  cerr << endl << "diag \\:\t";
  if (sq.diag1==MAGIC) cerr << "@";
  else {
    cerr << "0";
    res=false;
  }

  cerr << endl << "diag /:\t";
  if (sq.diag2==MAGIC) cerr << "@";
  else {
    cerr << "0";
    res=false;
  }
  cerr << endl;
  return res;
}

/**
 * Verify that a square is magic.
 *
 * @param sq the square to verify
 * @return true if verify ok
 */
bool verifyMagic2(square &sq) {
  int i;

  sumSquare(sq);                                   // calculate the sums

  for (int i=0; i<ORDER; i++) {
    if (sq.row[i]!=MAGIC) return false;
  }

  for (int i=0; i<ORDER; i++) {
    if (sq.col[i]!=MAGIC) return false;
  }

  if (sq.diag1!=MAGIC) return false;

  if (sq.diag2!=MAGIC) return false;
  return true;
}


/**
 * Permutate value for making diagonal \ magic without changing rows and
 * columns magic state.
 *
 * @param pos contains actual position in a column of choosed values
 * @param w an index for recursion
 * @param sq the square
 * @return true if ok
 */
bool permDiag1(int *pos, int w, square &sq) {
  int tmp;
  int n;

  if (w>=(ORDER*ORDER-1)) {
        square tmp;                                // tmp square

        for (int i=0; i<ORDER; i++) {
          for (int j=0; j<ORDER; j++) {            // build square
            tmp.cell[i][j]=PRIME[pos[i+j*ORDER]];
          }
        }
        if (verifyMagic2(tmp)) {
         sq=tmp;                                   // return the new square
         return true;
        }
  } else {
      for (n=w; n<ORDER*ORDER; n++) {              // calc. all the permutation
        tmp=pos[n];
        pos[n]=pos[w];
        pos[w]=tmp;
        if (permDiag1(pos, (w+1), sq))
          return true;
        tmp=pos[n];
        pos[n]=pos[w];
        pos[w]=tmp;
      }
    }
  return false;
}

/**
 * Makes the diagonal \ magic and see if diagonal / become magic, or repeat
 * forever.
 *
 * @param sq the square
 * @return true if diagonals are magics
 */
bool diagonal1(square &sq) {
  // position in a column of element to put into the diagonal
  int position[ORDER*ORDER]=UP_SEQ;

  return permDiag1(position, 0, sq);
}

/**
 * Apply method of magic solution to the passed prime square.
 *
 * @param sq the square with prime number
 * @return true if sucessful building
 */
bool magicC(square &sq) {
  return diagonal1(sq);
}

int main(int argc, char *argv[]) {
  // result of magic building
  bool result;

  // millisecond at start time
  long double startTime;

  // millisecond at end time
  long double endTime;

  startTime=actualTime();
  srand48((long int)startTime);

  eraseSquare(primeSquare);

  result=magicC(primeSquare);

  writeSquare(primeSquare);
  if (!verifyMagic(primeSquare))
        cout << endl << "There's no squares"
             << endl;

  // calculate and show statistic information
  endTime=actualTime();

  cerr << endl;
  cerr << "Total Working Time:\t\t" << ((endTime-startTime)/1000)
       << "\tseconds" << endl;
  cerr << "So in a second I can build:\t" << (1000/(endTime-startTime))
       << "\tsquares" << endl;
  cerr << "or in a minute I can build:\t" << (1000*60/(endTime-startTime))
       << "\tsquares" << endl;
  cerr << "in a hour I can build:\t\t" << (1000*60*60/(endTime-startTime))
       << "\tsquares" << endl;
  cerr << "in a day I can build:\t\t"
       << (1000*60*60/(endTime-startTime)*24)
       << "\tsquares" << endl;
  cerr << "in a year I can build:\t\t"
       << (1000*60*60/(endTime-startTime)*24*365)
       << "\tsquares" << endl;

  return 0;
}
